function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import fl from 'fantasy-land';
import * as bs from './index';

export var Stream = function () {

  // Example:
  //
  //    const myWrappedStream = new Stream(sink => {
  //      sink(1)
  //      retrun () => {}
  //    })
  //

  function Stream(basicStream) {
    _classCallCheck(this, Stream);

    // The wrapped BasicStream is available as `wrapped.observe`.
    // You can call it as a method in order to subscribe to the stream:
    //
    //    const usub = wrapped.observe(callback)
    //    unsub()
    //
    // Or you can use `wrapped.observe` to extract BasicStream from the wrapper e.g.:
    //
    //    const basicStream2 = map(x => x * 2)(wrapped.observe)
    //
    this.observe = bs.fromLoose(basicStream);
  }

  // Semigroup

  Stream.prototype.concat = function concat(other) {
    return new Stream(bs.merge([this.observe, other.observe]));
  };

  // Monoid

  Stream.empty = function empty() {
    return new Stream(bs.empty);
  };

  Stream.prototype.empty = function empty() {
    return Stream.empty();
  };

  // Functor

  Stream.prototype.map = function map(f) {
    return new Stream(bs.map(f)(this.observe));
  };

  // Apply

  Stream.prototype.ap = function ap(other) {
    // In order for .ap() to work `this` must be a `Stream<A => B>`,

    var streamF = this.observe;
    return new Stream(bs.ap(streamF)(other.observe));
  };

  // Applicative

  Stream.of = function of(x) {
    return new Stream(bs.just(x));
  };

  Stream.prototype.of = function of(x) {
    return Stream.of(x);
  };

  // Chain

  Stream.prototype.chain = function chain(f) {
    return new Stream(bs.chain(function (x) {
      return f(x).observe;
    })(this.observe));
  };

  // ------------------ Not from FL ------------------

  Stream.prototype.filter = function filter(f) {
    return new Stream(bs.filter(f)(this.observe));
  };

  Stream.prototype.scan = function scan(f, seed) {
    return new Stream(bs.scan(f, seed)(this.observe));
  };

  Stream.prototype.chainLatest = function chainLatest(f) {
    return new Stream(bs.chainLatest(function (x) {
      return f(x).observe;
    })(this.observe));
  };

  Stream.prototype.take = function take(n) {
    return new Stream(bs.take(n)(this.observe));
  };

  Stream.prototype.takeWhile = function takeWhile(f) {
    return new Stream(bs.takeWhile(f)(this.observe));
  };

  Stream.prototype.takeUntil = function takeUntil(other) {
    return new Stream(bs.takeUntil(other.observe)(this.observe));
  };

  Stream.prototype.skip = function skip(n) {
    return new Stream(bs.skip(n)(this.observe));
  };

  Stream.prototype.skipWhile = function skipWhile(f) {
    return new Stream(bs.skipWhile(f)(this.observe));
  };

  Stream.prototype.skipDuplicates = function skipDuplicates(f) {
    return new Stream(bs.skipDuplicates(f)(this.observe));
  };

  Stream.prototype.multicast = function multicast() {
    return new Stream(bs.multicast(this.observe));
  };

  Stream.prototype.startWith = function startWith(x) {
    return new Stream(bs.startWith(x)(this.observe));
  };

  Stream.prototype.transduce = function transduce(transducer) {
    return new Stream(bs.transduce(transducer)(this.observe));
  };

  Stream.merge = function merge(streams) {
    return new Stream(bs.merge(streams.map(function (x) {
      return x.observe;
    })));
  };

  Stream.map2 = function map2(f) {
    var lifted = bs.map2(f);
    return function (sA, sB) {
      return new Stream(lifted(sA.observe, sB.observe));
    };
  };

  Stream.map3 = function map3(f) {
    var lifted = bs.map3(f);
    return function (sA, sB, sC) {
      return new Stream(lifted(sA.observe, sB.observe, sC.observe));
    };
  };

  Stream.combineArray = function combineArray(arr) {
    return new Stream(bs.combineArray(arr.map(function (x) {
      return x.observe;
    })));
  };

  Stream.combineObject = function combineObject(obj) {
    var ofBasicStreams = {};
    Object.keys(obj).forEach(function (key) {
      ofBasicStreams[key] = obj[key].observe;
    });
    return new Stream(bs.combineObject(ofBasicStreams));
  };

  return Stream;
}();

// Flow doesn't support computed method names,
// we have to bypass type checking for FL methods :(
//
function addFlMethods(constructor, proto) {
  ['concat', 'empty', 'map', 'ap', 'of', 'chain'].forEach(function (name) {
    proto[fl[name]] = proto[name];
  });['empty', 'of'].forEach(function (name) {
    constructor[fl[name]] = constructor[name];
  });
}
addFlMethods(Stream, Stream.prototype);